@echo off
setlocal enabledelayedexpansion

set OUTPUT_FILE=%~1
set PLATFORM=%2
set VERSION_BASE=%3

set OUTPUT_FILE_BASENAME=%~n1

REM Validate parameters
set PLATFORM_RECOGNIZED=
if "%PLATFORM%" == "x86" set PLATFORM_RECOGNIZED=1
if "%PLATFORM%" == "x64" set PLATFORM_RECOGNIZED=1

set ARGS_CORRECT=1
if not defined OUTPUT_FILE set ARGS_CORRECT=
if not defined PLATFORM_RECOGNIZED set ARGS_CORRECT=
if not defined VERSION_BASE set ARGS_CORRECT=
if not defined ARGS_CORRECT (
  echo Usage: %0 OUTPUT_FILE ARCH VERSION_BASE
  echo Where ARCH is "x86" or "x64"
  exit /b 1
)

REM Find WiX difxapp library
SET SOURCES_PATH=%~dp0
for %%i in (light.exe) do set WIX_BIN_PATH=%%~dp$PATH:i

if "%WIX_BIN_PATH%" == "" (
  echo Error: Unable to find WiX in path
  exit /b 1
)

set DIFXAPP_PATH=%WIX_BIN_PATH%difxapp_%PLATFORM%.wixlib
if not EXIST "%DIFXAPP_PATH%" (
  echo Error: Unable to find difxapp path
  exit /b 1
)

REM Process the build
if EXIST build (
  echo Error: 'build' file or directory exists
  exit /b 1
)

mkdir build || (echo Error: Failed to create 'build' directory & exit /b 1)

candle %SOURCES_PATH%wix/AetpkssReg.wxs -dPlatform=%PLATFORM% -out build/AetpkssReg.wixobj && ^
candle %SOURCES_PATH%wix/CardModuleReg.wxs -dPlatform=%PLATFORM% -out build/CardModuleReg.wixobj && ^
candle %SOURCES_PATH%wix/JavaCardDataReg.wxs -dPlatform=%PLATFORM% -out build/JavaCardDataReg.wixobj && ^
candle %SOURCES_PATH%wix/ProductInfoReg.wxs -dPlatform=%PLATFORM% -out build/ProductInfoReg.wixobj && ^
candle %SOURCES_PATH%wix/CardProfilesReg.wxs -dPlatform=%PLATFORM% -out build/CardProfilesReg.wixobj && ^
candle %SOURCES_PATH%wix/SafeSignReg.wxs -dPlatform=%PLATFORM% -out build/SafeSignReg.wixobj && ^
candle %SOURCES_PATH%wix/TokenAdminReg.wxs -dPlatform=%PLATFORM% -out build/TokenAdminReg.wixobj && ^
candle %SOURCES_PATH%wix/ReadOnlyMinidriverInstall.wxs -dPlatform=%PLATFORM% -out build/ReadOnlyMinidriverInstall.wixobj && ^
candle %SOURCES_PATH%wix/ReadWriteMinidriverInstall.wxs -dPlatform=%PLATFORM% -out build/ReadWriteMinidriverInstall.wixobj && ^
candle %SOURCES_PATH%wix/DirectoryStructure.wxs -dPlatform=%PLATFORM% -out build/DirectoryStructure.wixobj && ^
candle %SOURCES_PATH%wix/InternalComponents.wxs -dPlatform=%PLATFORM% -out build/InternalComponents.wixobj && ^
candle %SOURCES_PATH%wix/Pkcs11.wxs -dPlatform=%PLATFORM% -out build/Pkcs11.wixobj && ^
candle -ext WixDifxAppExtension %SOURCES_PATH%wix/Minidriver.wxs -dPlatform=%PLATFORM% -out build/Minidriver.wixobj && ^
candle %SOURCES_PATH%wix/Utilities.wxs -dPlatform=%PLATFORM% -out build/Utilities.wixobj && ^
candle %SOURCES_PATH%wix/Locales.wxs -dPlatform=%PLATFORM% -out build/Locales.wixobj && ^
candle %SOURCES_PATH%wix/Documentation.wxs -dPlatform=%PLATFORM% -out build/Documentation.wixobj && ^
candle %SOURCES_PATH%wix/Main.wxs -dPlatform=%PLATFORM% -dVersionBase=%VERSION_BASE% -out build/Main.wixobj
if ERRORLEVEL 1 (
  echo Error: Failed to process WiX source files
  rmdir /S /Q build
  exit /b 1
)

REM Produce MSI's for all languages
set DEFAULT_LANGUAGE=en-US
set LANGUAGES=(en-US,ca-ES,cs-CZ,de-DE,es-ES,fi-FI,fr-FR,hr-HR,hu-HU,it-IT,ja-JP,ko-KR,nl-NL,pt-BR,pt-PT,ru-RU,sr-latn-cs,sr-cyrl-cs,th-TH,tr-TR,zh-CN,zh-TW)
set WIX_UI_TRANSLATIONS=(sr-cyrl-cs)
for %%I in %LANGUAGES% do (
  echo Building language: %%I

  for %%J in %WIX_UI_TRANSLATIONS% do (
    if %%I == %%J (
      SET WIX_UI_LOC= -loc "%SOURCES_PATH%wix/InstallerLocale/WixUI_%%I.wxl"
    )
  )

  light -reusecab -cc build/cache -ext WixDifxAppExtension -ext WixUIExtension "%DIFXAPP_PATH%" ^
    -b %SOURCES_PATH% -o "build/%OUTPUT_FILE_BASENAME%.%%I.msi" -spdb ^
    -cultures:%%I ^
    -loc "%SOURCES_PATH%wix/InstallerLocale/%%I.wxl" !WIX_UI_LOC! ^
    build/AetpkssReg.wixobj ^
    build/CardModuleReg.wixobj ^
    build/JavaCardDataReg.wixobj ^
    build/ProductInfoReg.wixobj ^
    build/CardProfilesReg.wixobj ^
    build/SafeSignReg.wixobj ^
    build/TokenAdminReg.wixobj ^
    build/ReadOnlyMinidriverInstall.wixobj ^
    build/ReadWriteMinidriverInstall.wixobj ^
    build/DirectoryStructure.wixobj ^
    build/InternalComponents.wixobj ^
    build/Pkcs11.wixobj ^
    build/Minidriver.wixobj ^
    build/Utilities.wixobj ^
    build/Locales.wixobj ^
    build/Documentation.wixobj ^
    build/Main.wixobj
  if ERRORLEVEL 1 (
    echo Error: Failed to link installer
    rmdir /S /Q build
    exit /b 1
  )
)

copy /Y "build\%OUTPUT_FILE_BASENAME%.%DEFAULT_LANGUAGE%.msi" "%OUTPUT_FILE%"

REM Produce the MST's for the non-default languages
for %%I in %LANGUAGES% do (
  if not "%%I" == "%DEFAULT_LANGUAGE%" (
    echo Producing mst for language: %%I
    torch -p -t language "%OUTPUT_FILE%" "build/%OUTPUT_FILE_BASENAME%.%%I.msi" -out %%I.mst

    if ERRORLEVEL 1 (
      echo Error: Failed to create installer mst
      rmdir /S /Q build
      exit /b 1
    )
  )
)

echo Created installer successfully!
rmdir /S /Q build
